@@ -142,6 +142,16 @@ local_bru_safe_inla <- function(multicore = FALSE,
                                 quietly = TRUE,
                                 envir = parent.frame()) {
   if (requireNamespace("INLA", quietly = TRUE)) {
+    inla.call <- tryCatch(
+      INLA::inla.getOption("inla.call"),
+      error = function(e) {
+        e
+      }
+    )
+    if (inherits(inla.call, "simpleError")) {
+      return(testthat::skip("inla.getOption('inla.call') failed, skip INLA tests."))
+    }
+
     # Save the num.threads option so it can be restored
     old_threads <- tryCatch(
       INLA::inla.getOption("num.threads"),
@@ -174,7 +184,29 @@ local_bru_safe_inla <- function(multicore = FALSE,
       )
       INLA::inla.setOption(fmesher.evolution = max(old_fmesher_evolution, 2L))
     }
+
+    if ("fmesher.evolution.warn" %in% names(INLA::inla.getOption())) {
+      # Save the fmesher.evolution.warn option so it can be restored
+      old_fmesher_evolution_warn <- INLA::inla.getOption("fmesher.evolution.warn")
+      withr::defer(
+        INLA::inla.setOption(fmesher.evolution.warn = old_fmesher_evolution_warn),
+        envir
+      )
+      INLA::inla.setOption(fmesher.evolution.warn = TRUE)
+    }
+
+    if ("fmesher.evolution.verbosity" %in% names(INLA::inla.getOption())) {
+      # Save the fmesher.evolution.verbosity option so it can be restored
+      old_fmesher_evolution_verbosity <- INLA::inla.getOption("fmesher.evolution.verbosity")
+      withr::defer(
+        INLA::inla.setOption(fmesher.evolution.verbosity = old_fmesher_evolution_verbosity),
+        envir
+      )
+      INLA::inla.setOption(fmesher.evolution.verbosity = "stop")
+    }
+    # withr::local_options(lifecycle_verbosity = "quiet", .local_envir = envir)
   }
+
   if (!multicore) {
     local_bru_options_set(num.threads = "1:1", envir = envir)
   }
